/** Define Kerrighed Capabilities (not exported outside kernel)
 * @author Jean Parpaillon (c) Inria 2006
 */

#ifndef _KERRIGHED_CAPABILITIES_H_INTERNAL
#define _KERRIGHED_CAPABILITIES_H_INTERNAL

#include <linux/capability.h>
#include "sys/capabilities.h"

/*
 * MACROS
 */
#define __KRG_CAP_SUPPORTED_BASE CAP_TO_MASK(CAP_CHANGE_KERRIGHED_CAP)
#ifdef CONFIG_CLUSTER_WIDE_PROC_INFRA
#define __KRG_CAP_SUPPORTED_PROCFS CAP_TO_MASK(CAP_SEE_LOCAL_PROC_STAT)
#else
#define __KRG_CAP_SUPPORTED_PROCFS 0
#endif
#ifdef CONFIG_KRG_MM
#define __KRG_CAP_SUPPORTED_MM CAP_TO_MASK(CAP_USE_REMOTE_MEMORY)
#else
#define __KRG_CAP_SUPPORTED_MM 0
#endif
#ifdef CONFIG_KRG_EPM
#define __KRG_CAP_SUPPORTED_EPM CAP_TO_MASK(CAP_CAN_MIGRATE)	 \
				|CAP_TO_MASK(CAP_DISTANT_FORK)   \
				|CAP_TO_MASK(CAP_CHECKPOINTABLE)
#else
#define __KRG_CAP_SUPPORTED_EPM 0
#endif
#ifdef CONFIG_KRG_DEBUG
#define __KRG_CAP_SUPPORTED_DEBUG CAP_TO_MASK(CAP_DEBUG)
#else
#define __KRG_CAP_SUPPORTED_DEBUG 0
#endif
#ifdef CONFIG_KRG_SYSCALL_EXIT_HOOK
#define __KRG_CAP_SUPPORTED_SEH CAP_TO_MASK(CAP_SYSCALL_EXIT_HOOK)
#else
#define __KRG_CAP_SUPPORTED_SEH 0
#endif
#define KRG_CAP_SUPPORTED to_cap_t(__KRG_CAP_SUPPORTED_BASE    \
				   |__KRG_CAP_SUPPORTED_PROCFS \
				   |__KRG_CAP_SUPPORTED_MM     \
				   |__KRG_CAP_SUPPORTED_EPM    \
				   |__KRG_CAP_SUPPORTED_DEBUG  \
				   |__KRG_CAP_SUPPORTED_SEH)

#define KRG_CAP_INIT_PERM_SET KRG_CAP_SUPPORTED
#define KRG_CAP_INIT_EFF_SET to_cap_t(CAP_TO_MASK(CAP_CHANGE_KERRIGHED_CAP))
#define KRG_CAP_INIT_INH_PER_SET KRG_CAP_INIT_PERM_SET
#define KRG_CAP_INIT_INH_EFF_SET KRG_CAP_INIT_EFF_SET

#define KRG_CAP_INIT_ATOMIC_COUNTERS \
	{{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0}}

#define can_use_krg_cap(task,cap) \
	(cap_raised(task->krg_cap_effective,cap) \
	 && !atomic_read(&task->krg_cap_unavailable[cap]) \
	 && !atomic_read(&task->krg_cap_unavailable_private[cap]))

#define DEFAULT_KRG_DAEMON_CAP_EFF to_cap_t(0)
#define DEFAULT_KRG_DAEMON_CAP_PERM to_cap_t(0)

/*
 * TYPES
 */
#ifdef __KERNEL__

struct krg_cap_struct
{
	kernel_cap_t krg_cap_effective;
	kernel_cap_t krg_cap_permitted;
	kernel_cap_t krg_cap_inheritable_permitted;
	kernel_cap_t krg_cap_inheritable_effective;
};

#else

#include <sys/types.h>

#endif /* __KERNEL__ */

struct krg_cap_pid_desc
{
	pid_t pid;
	krg_cap_t *caps;
};

#endif /* _KERRIGHED_CAPABILITIES_H_INTERNAL */
