#ifndef __KKRG_SCHED_H__
#define __KKRG_SCHED_H__

#ifdef CONFIG_KRG_EPM

#include <linux/sched.h>
#include <linux/types.h>
#include <kerrighed/sys/types.h>

/* WARNING: these flags must not overwrite the flags defined in linux/sched.h
 * The place is becoming sparse ... */

#define PF_EXIT_NOTIFYING  0x00000008  /* I am exiting and notifying my remote
					* parent */
#define PF_DISTANT_FORKING 0x04000000  /* I am doing a distant fork */
#define PF_MIGRATING       0x08000000  /* I am migrating */
#define PF_CHECKPOINTING   0x40000000  /* I am doing a checkpoint */
#define PF_AWAY		   0x80000000  /* I don't want to be considered as local
					* by my relatives */

#define PF_KERRIGHED_ACTION (PF_DISTANT_FORKING | PF_MIGRATING	\
			     | PF_CHECKPOINTING)

/*
 * Nests inside and outside of read_lock(&taskslist_lock), but neither inside
 * nor outside write_lock(_irq)(&tasklist_lock).
 * Nests outside sighand->lock.
 */
extern rwlock_t krg_action_lock;

static inline void krg_action_block_all(void)
{
	read_lock(&krg_action_lock);
}

static inline void krg_action_unblock_all(void)
{
	read_unlock(&krg_action_lock);
}

struct task_struct;

static inline int krg_action_any_pending(struct task_struct *task)
{
	return (task->flags & PF_KERRIGHED_ACTION);
}

static inline int krg_action_block_any(struct task_struct *task)
{
	int pending;

	krg_action_block_all();
	pending = krg_action_any_pending(task);
	if (pending)
		krg_action_unblock_all();
	return !pending;
}

static inline void krg_action_unblock_any(struct task_struct *task)
{
	krg_action_unblock_all();
}

struct pt_regs;
struct completion;
struct children_kddm_object;
struct siginfo;
struct rusage;

extern struct task_struct *baby_sitter;

/* remote clone */
extern int (*kh_do_fork)(unsigned long clone_flags,
			 unsigned long stack_start,
			 struct pt_regs *regs,
			 unsigned long stack_size,
			 int *parent_tidptr,
			 int *child_tidptr);

/* vfork */
extern void (*kh_vfork_done)(struct completion *vfork_done);

/* pgrp */
extern int (*kh_will_become_orphaned_pgrp)(pid_t pgrp, pid_t ignored_kid);

/* exit */
extern int (*kh_do_notify_parent)(struct task_struct *tsk,
				  struct siginfo *info);
extern int (*kh_do_wait)(struct children_kddm_object *obj,
			 pid_t pid, int options, struct siginfo __user *infop,
			 int __user *stat_addr, struct rusage __user *ru);
extern void (*kh_delay_release_task)(struct task_struct *task);
int wait_task_zombie(struct task_struct *p, int noreap,
		     struct siginfo __user *infop,
		     int __user *stat_addr, struct rusage __user *ru);

/* checkpoint */
extern int (*kh_copy_application)(struct task_struct *task);
extern void (*kh_exit_application)(struct task_struct *task);

/* migration */
fastcall NORET_TYPE void do_exit_wo_notify(long error_code)
	ATTRIB_NORET;

#endif /* CONFIG_KRG_EPM */

#ifdef CONFIG_KRG_SCHED
#ifdef CONFIG_KRG_SCHED_LEGACY
extern void (*kh_calc_load)(unsigned long ticks);
extern void (*kh_process_on)(struct task_struct *tsk);
extern void (*kh_process_off)(struct task_struct *tsk);
#endif /* CONFIG_KRG_SCHED_LEGACY */

#ifdef CONFIG_KRG_SCHED_CONFIG
#include <kerrighed/module_hook.h>

extern struct module_hook_desc kmh_calc_load;
extern struct module_hook_desc kmh_process_on;
extern struct module_hook_desc kmh_process_off;
#endif /* CONFIG_KRG_SCHED_CONFIG */

extern int (*kh_copy_sched_info)(struct task_struct *tsk);
extern void (*kh_free_sched_info)(struct task_struct *tsk);
#endif /* CONFIG_KRG_SCHED */

#endif /* __KKRG_SCHED_H__ */
