#ifndef __KKRG_SEM__
#define __KKRG_SEM__

#include <kerrighed/ipc.h>
#include <linux/ipc_util.h>

/** Kerrighed Hooks **/

extern struct sem_array * (*kh_ipcsem_lock)(struct ipc_namespace *ns, int id);
extern void (*kh_ipcsem_unlock)(struct sem_array *sma);

extern int (*kh_ipcsem_newary)(struct ipc_namespace *ns,
			       struct sem_array *sma);
extern void (*kh_ipcsem_freeary)(struct ipc_namespace *ns,
				 struct sem_array *sma, int id);

extern void (*kh_ipcsem_wakeup_process)(struct sem_queue *q, int error);

extern int (*kh_ipcsem_copy_semundo)(unsigned long clone_flags,
				     struct task_struct *tsk);

extern struct sem_undo * (*kh_ipcsem_find_undo)(struct sem_array* sma);

extern void (*kh_ipcsem_exit_sem)(struct task_struct *tsk);

#define sem_ids(ns)	(*((ns)->ids[IPC_SEM_IDS]))

static inline struct sem_array* local_sem_lock(struct ipc_namespace *ns,
					       int id)
{
	return (struct sem_array*)ipc_lock(&sem_ids(ns),id);
}

static inline void local_sem_unlock(struct sem_array *sma)
{
	ipc_unlock(&(sma)->sem_perm);
}

static inline struct sem_array* sem_lock(struct ipc_namespace *ns, int id)
{
	if (kh_ipcsem_lock)
		return kh_ipcsem_lock(ns, id);
	else
		return local_sem_lock(ns, id);
}


static inline void sem_unlock(struct sem_array *sma)
{
	if (kh_ipcsem_unlock)
		kh_ipcsem_unlock(sma);
	else
		local_sem_unlock(sma);
}

extern void append_to_remote_queue(struct sem_array * sma,
				   struct sem_queue *q);

extern void remove_from_remote_queue(struct sem_array * sma,
				     struct sem_queue *q,
				     int free);

extern void __remove_from_queue(struct sem_array * sma,
				struct sem_queue *q);

extern void __freeary(struct ipc_namespace *ns, struct sem_array *sma, int id);

extern void __exit_sem_found(struct sem_array *sma, struct sem_undo *u);

#endif // __KKRG_SEM__
